// Copyright (c) 2017-2020 LG Electronics, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// SPDX-License-Identifier: Apache-2.0

#ifndef NEPALI_AUTOMATA_H
#define NEPALI_AUTOMATA_H

#include "automatabase.h"
#include <QLocale>

class NepaliAutomata : public AutomataBase
{

public:
    NepaliAutomata()
    {
        m_keymap.clear();
        addSymbolKeys();
        addNumberKeys();
        addShiftKeys();
        addNormalKeys();
        addAccentKeys();
        addAltKeysForHID();
        addAltShiftedKeys();
    }

    void setPreedit(const QString &preeditString, int cursorPos)
    {
        Q_UNUSED(cursorPos);
        m_surroundingText = preeditString;
    }

    bool isPredictionSupported()
    {
        return false;
    }

    static AutomataBase* create()
    {
        return new NepaliAutomata();
    }
private:
    void addSymbolKeys()
    {
        m_keymap.insert(0x21,QString(QChar(0x0967)));
        m_keymap.insert(0x40,QString(QChar(0x0968)));
        m_keymap.insert(0x23,QString(QChar(0x0969)));
        m_keymap.insert(0x24,QString(QChar(0x096a)));
        m_keymap.insert(0x25,QString(QChar(0x096b)));
        m_keymap.insert(0x5E,QString(QChar(0x096c)));
        m_keymap.insert(0x26,QString(QChar(0x096d)));
        m_keymap.insert(0x2A,QString(QChar(0x096e)));
        m_keymap.insert(0x28,QString(QChar(0x096f)));
        m_keymap.insert(0x29,QString(QChar(0x0966)));
        m_keymap.insert(0x5F,QString(QChar(0x0029)));
        m_keymap.insert(0x2B,QString(QChar(0x0902)));
        m_keymap.insert(0x2D,QString(QChar(0x28)));
        m_keymap.insert(0x3D,QString(QChar(0x2E)));
    }

    void addNumberKeys()
    {
        m_keymap.insert(0x30,QString(QChar(0x0923)));
        m_keymap.insert(0x31,QString("\u091c\u094d\u091e"));
        m_keymap.insert(0x32,QString(QChar(0x0918)));
        m_keymap.insert(0x33,QString(QChar(0x0919)));
        m_keymap.insert(0x34,QString(QChar(0x091d)));
        m_keymap.insert(0x35,QString(QChar(0x091b)));
        m_keymap.insert(0x36,QString(QChar(0x091f)));
        m_keymap.insert(0x37,QString(QChar(0x0920)));
        m_keymap.insert(0x38,QString(QChar(0x0921)));
        m_keymap.insert(0x39,QString(QChar(0x0922)));
    }

    void addShiftKeys()
    {
        m_keymap.insert(0x51,QString(QChar(0x094b)));
        m_keymap.insert(0x57,QString("\u0927\u094d"));
        m_keymap.insert(0x45,QString("\u092d\u094d"));
        m_keymap.insert(0x52,QString("\u091a\u094d"));
        m_keymap.insert(0x54,QString("\u0924\u094d"));
        m_keymap.insert(0x59,QString("\u0925\u094d"));
        m_keymap.insert(0x55,QString("\u0917\u094d"));
        m_keymap.insert(0x49,QString("\u0915\u094d\u0937"));
        m_keymap.insert(0x4F,QString(QChar(0x0907)));
        m_keymap.insert(0x50,QString(QChar(0x090f)));
        m_keymap.insert(0x7B,QString("\u0930\u094d"));
        m_keymap.insert(0x7D,QString(QChar(0x0948)));
        m_keymap.insert(0x41,QString("\u092c\u094d"));
        m_keymap.insert(0x53,QString("\u0915\u094d"));
        m_keymap.insert(0x44,QString("\u092e\u094d"));
        m_keymap.insert(0x46,QString(QChar(0x0901)));
        m_keymap.insert(0x47,QString("\u0928\u094d"));
        m_keymap.insert(0x48,QString("\u091c\u094d"));
        m_keymap.insert(0x4A,QString("\u0935\u094d"));
        m_keymap.insert(0x4B,QString("\u092a\u094d"));
        m_keymap.insert(0x4C,QString(QChar(0x0940)));
        m_keymap.insert(0x3A,QString("\u0938\u094d"));
        m_keymap.insert(0x22,QString(QChar(0x0942)));
        m_keymap.insert(0xFE,QString(QChar(0x094d))); //also in lower case
        m_keymap.insert(0x5A,QString("\u0936\u094d"));
        m_keymap.insert(0x58,QString("\u0939\u094d"));
        m_keymap.insert(0x43,QString(QChar(0x090b)));
        m_keymap.insert(0x56,QString("\u0916\u094d"));
        m_keymap.insert(0x42,QString("\u0926\u094d"));
        m_keymap.insert(0x4E,QString("\u0932\u094d"));
        m_keymap.insert(0x4D,QString(QChar(0x0903)));
        m_keymap.insert(0x3C,QString(QChar(0x003f)));
        m_keymap.insert(0x3E,QString("\u0936\u094d\u0930"));
        m_keymap.insert(0x3F,QString("\u0930\u0942"));
    }

    void addNormalKeys()
    {
        m_keymap.insert(0x71,QString("\u0924\u094d\u0930"));
        m_keymap.insert(0x77,QString(QChar(0x0927)));
        m_keymap.insert(0x65,QString(QChar(0x092D)));
        m_keymap.insert(0x72,QString(QChar(0x091A)));
        m_keymap.insert(0x74,QString(QChar(0x0924)));
        m_keymap.insert(0x79,QString(QChar(0x0925)));
        m_keymap.insert(0x75,QString(QChar(0x0917)));
        m_keymap.insert(0x69,QString(QChar(0x0937)));
        m_keymap.insert(0x6F,QString(QChar(0x092F)));
        m_keymap.insert(0x70,QString(QChar(0x0909)));
        m_keymap.insert(0x5B,QString(QChar(0x0943)));
        m_keymap.insert(0x5D,QString(QChar(0x0947)));
        m_keymap.insert(0x61,QString(QChar(0x092c)));
        m_keymap.insert(0x73,QString(QChar(0x0915)));
        m_keymap.insert(0x64,QString(QChar(0x092e)));
        m_keymap.insert(0x66,QString(QChar(0x093e)));
        m_keymap.insert(0x67,QString(QChar(0x0928)));
        m_keymap.insert(0x68,QString(QChar(0x091c)));
        m_keymap.insert(0x6A,QString(QChar(0x0935)));
        m_keymap.insert(0x6B,QString(QChar(0x092a)));
        m_keymap.insert(0x6C,QString(QChar(0x093f)));
        m_keymap.insert(0x3B,QString(QChar(0x0938)));
        m_keymap.insert(0x27,QString(QChar(0x0941)));
        m_keymap.insert(0xC4,QString("\u094d\u0930"));
        m_keymap.insert(0x7A,QString(QChar(0x0936)));
        m_keymap.insert(0x78,QString(QChar(0x0939)));
        m_keymap.insert(0x63,QString(QChar(0x0905)));
        m_keymap.insert(0x76,QString(QChar(0x0916)));
        m_keymap.insert(0x62,QString(QChar(0x0926)));
        m_keymap.insert(0x6E,QString(QChar(0x0932)));
        m_keymap.insert(0x6D,QString(QChar(0x092b)));
        m_keymap.insert(0x2C,QString(QChar(0x002c)));
        m_keymap.insert(0x2E,QString(QChar(0x0964)));
        m_keymap.insert(0x2F,QString(QChar(0x0930)));
        m_keymap.insert(0x20,QString(QChar(0x20)));
    }

     void addAccentKeys()
     {
		 //lower case
         m_keymap.insert(0x60,QString(QChar(0x091e)));
                 //upper case
         m_keymap.insert(0x7E,QString("\u091e\u094d"));
     }

     void addAltKeysForHID()
     {
         m_keymap.insert(0xA0,QString(QChar(0x31)));
         m_keymap.insert(0xA1,QString(QChar(0x32)));
         m_keymap.insert(0xA2,QString(QChar(0x33)));
         m_keymap.insert(0xA3,QString(QChar(0x34)));
         m_keymap.insert(0xA4,QString(QChar(0x35)));
         m_keymap.insert(0xA5,QString(QChar(0x36)));
         m_keymap.insert(0xA6,QString(QChar(0x37)));
         m_keymap.insert(0xA7,QString(QChar(0x38)));
         m_keymap.insert(0xA8,QString(QChar(0x39)));
         m_keymap.insert(0xA9,QString(QChar(0x30)));
         m_keymap.insert(0xAB,QString(QChar(0x2B)));
         m_keymap.insert(0xB5,QString(QChar(0x090A)));
         m_keymap.insert(0xB7,QString(QChar(0x0914)));
         m_keymap.insert(0xC7,QString(QChar(0x0906)));
     }

     void addAltShiftedKeys()
     {
         m_keymap.insert(0xDA,QString(QChar(0x21)));
         m_keymap.insert(0xDB,QString(QChar(0x40)));
         m_keymap.insert(0xDC,QString(QChar(0x23)));
         m_keymap.insert(0xDD,QString(QChar(0x24)));
         m_keymap.insert(0xDE,QString(QChar(0x25)));
         m_keymap.insert(0xDF,QString(QChar(0x5E)));
         m_keymap.insert(0xE0,QString(QChar(0x26)));
         m_keymap.insert(0xE1,QString(QChar(0x2A)));
         m_keymap.insert(0xE2,QString(QChar(0x27)));
         m_keymap.insert(0xE3,QString(QChar(0x22)));
         m_keymap.insert(0xE4,QString(QChar(0x5F)));
         m_keymap.insert(0xE5,QString(QChar(0x2B)));
         m_keymap.insert(0xE6,QString(QChar(0x094c)));
         m_keymap.insert(0xEE,QString(QChar(0x0908)));
         m_keymap.insert(0xEF,QString(QChar(0x0910)));
         m_keymap.insert(0xF1,QString(QChar(0x0913)));
     }

};
#endif // NEPALI_AUTOMATA_H

